@@ -1,75 +1,94 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""Some classes used to handle advanced configuration in simple to
-complex applications.
+"""Classes to handle advanced configuration in simple to complex applications.
 
-It's able to load the configuration from a file and or command line
-options, to generate a sample configuration file or to display program's
-usage. It basically fill the gap between optik/optparse and ConfigParser,
-with some additional data types (available as standalone optik extension
-in the `optik_ext` module)
+Allows to load the configuration from a file or from command line
+options, to generate a sample configuration file or to display
+program's usage. Fills the gap between optik/optparse and ConfigParser
+by adding data types (which are also available as a standalone optik
+extension in the `optik_ext` module).
 
 
 Quick start: simplest usage
-```````````````````````````
+---------------------------
 
-import sys
-from logilab.common.configuration import Configuration
+.. python ::
 
-options = [('dothis', {'type':'yn', 'default': True, 'metavar': '<y or n>'}),
-           ('value', {'type': 'string', 'metavar': '<string>'}),
-           ('multiple', {'type': 'csv', 'default': ('yop',),
-                         'metavar': '<comma separated values>',
-                         'help': 'you can also document the option'}),
-           ('number', {'type': 'int', 'default':2, 'metavar':'<int>'}),
-           ]
-config = Configuration(options=options, name='My config')
-print config['dothis']
-print config['value']
-print config['multiple']
-print config['number']
-
-print config.help()
-
-f = open('myconfig.ini', 'w')
-f.write('''[MY CONFIG]
-number = 3
-dothis = no
-multiple = 1,2,3
-''')
-f.close()
-config.load_file_configuration('myconfig.ini')
-print config['dothis']
-print config['value']
-print config['multiple']
-print config['number']
-
-sys.argv = ['mon prog', '--value', 'bacon', '--multiple', '4,5,6',
-            'nonoptionargument']
-print config.load_command_line_configuration()
-print config['value']
-
-config.generate_config()
+  >>> import sys
+  >>> from logilab.common.configuration import Configuration
+  >>> options = [('dothis', {'type':'yn', 'default': True, 'metavar': '<y or n>'}),
+  ...            ('value', {'type': 'string', 'metavar': '<string>'}),
+  ...            ('multiple', {'type': 'csv', 'default': ('yop',),
+  ...                          'metavar': '<comma separated values>',
+  ...                          'help': 'you can also document the option'}),
+  ...            ('number', {'type': 'int', 'default':2, 'metavar':'<int>'}),
+  ...           ]
+  >>> config = Configuration(options=options, name='My config')
+  >>> print config['dothis']
+  True
+  >>> print config['value']
+  None
+  >>> print config['multiple']
+  ('yop',)
+  >>> print config['number']
+  2
+  >>> print config.help()
+  Usage:  [options]
+  
+  Options:
+    -h, --help            show this help message and exit
+    --dothis=<y or n>     
+    --value=<string>      
+    --multiple=<comma separated values>
+                          you can also document the option [current: none]
+    --number=<int>        
+  
+  >>> f = open('myconfig.ini', 'w')
+  >>> f.write('''[MY CONFIG]
+  ... number = 3
+  ... dothis = no
+  ... multiple = 1,2,3
+  ... ''')
+  >>> f.close()
+  >>> config.load_file_configuration('myconfig.ini')
+  >>> print config['dothis']
+  False
+  >>> print config['value']
+  None
+  >>> print config['multiple']
+  ['1', '2', '3']
+  >>> print config['number']
+  3
+  >>> sys.argv = ['mon prog', '--value', 'bacon', '--multiple', '4,5,6',
+  ...             'nonoptionargument']
+  >>> print config.load_command_line_configuration()
+  ['nonoptionargument']
+  >>> print config['value']
+  bacon
+  >>> config.generate_config()
+  # class for simple configurations which don't need the
+  # manager / providers model and prefer delegation to inheritance
+  # 
+  # configuration values are accessible through a dict like interface
+  # 
+  [MY CONFIG]
+  
+  dothis=no
+  
+  value=bacon
+  
+  # you can also document the option
+  multiple=4,5,6
+  
+  number=3
+  >>>
 
 
-:author:    Logilab
-:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
+:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
-
 from __future__ import generators 
-
 __docformat__ = "restructuredtext en"
+
 __all__ = ('OptionsManagerMixIn', 'OptionsProviderMixIn',
            'ConfigurationMixIn', 'Configuration',
            'OptionsManager2ConfigurationAdapter')
@@ -77,7 +96,7 @@ __all__ = ('OptionsManagerMixIn', 'OptionsProviderMixIn',
 import os
 import sys
 import re
-from os.path import exists
+from os.path import exists, expanduser
 from copy import copy
 from ConfigParser import ConfigParser, NoOptionError, NoSectionError
 
@@ -143,7 +162,7 @@ def password_validator(opt_dict, name, value):
 
 def date_validator(opt_dict, name, value):
     """validate and return a mx DateTime object for option of type 'date'"""
-    return check_password(None, name, value)
+    return check_date(None, name, value)
 
 
 VALIDATORS = {'string' : unquote,
@@ -164,7 +183,7 @@ VALIDATORS = {'string' : unquote,
               }
 
 def _call_validator(opttype, optdict, option, value):
-    if not VALIDATORS.has_key(opttype):
+    if opttype not in VALIDATORS:
         raise Exception('Unsupported type "%s"' % opttype)
     try:
         return VALIDATORS[opttype](optdict, option, value)
@@ -300,7 +319,9 @@ def rest_format_section(stream, section, options, doc=None):
         if help:
             print >> stream, normalize_text(help, line_len=79, indent='  ')
         if value:
-            print >> stream, '  Default: %s' % format_option_value(optdict, value)
+            print >> stream, ''
+            print >> stream, '  Default: ``%s``' % str(
+                format_option_value(optdict, value)).replace("`` ","```` ``")
 
 
 class OptionsManagerMixIn(object):
@@ -337,16 +358,17 @@ class OptionsManagerMixIn(object):
         else:
             self.options_providers.append(provider)
         non_group_spec_options = [option for option in provider.options
-                                  if not option[1].has_key('group')]
+                                  if 'group' not in option[1]]
         groups = getattr(provider, 'option_groups', ())
         if own_group:
             self.add_option_group(provider.name.upper(), provider.__doc__,
                                   non_group_spec_options, provider)
         else:
             for opt_name, opt_dict in non_group_spec_options:
-                args, opt_dict = self.optik_option(provider, opt_name, opt_dict)
+                args, opt_dict = self.optik_option(provider, opt_name, opt_dict)                
                 self._optik_parser.add_option(*args, **opt_dict)
-                self._all_options[opt_name] = provider                
+                
+                self._all_options[opt_name] = provider
         for gname, gdoc in groups:
             goptions = [option for option in provider.options
                         if option[1].get('group') == gname]
@@ -356,7 +378,8 @@ class OptionsManagerMixIn(object):
         """add an option group including the listed options
         """
         # add section to the config file
-        self._config_parser.add_section(group_name)
+        if group_name != "DEFAULT":
+            self._config_parser.add_section(group_name)
         # add option group to the command line parser
         if options:
             group = OptionGroup(self._optik_parser,
@@ -373,23 +396,24 @@ class OptionsManagerMixIn(object):
         use with optik/optparse
         """
         opt_dict = copy(opt_dict)
-        if opt_dict.has_key('action'):
+        if 'action' in opt_dict:
             self._nocallback_options[provider] = opt_name
         else:
             opt_dict['action'] = 'callback'
             opt_dict['callback'] = self.cb_set_provider_option
-        for specific in ('default', 'group', 'inputlevel'):
-            if opt_dict.has_key(specific):
-                del opt_dict[specific]
-                if (OPTPARSE_FORMAT_DEFAULT
-                    and specific == 'default' and opt_dict.has_key('help')):
-                    opt_dict['help'] += ' [current: %default]'
+        # default is handled here and *must not* be given to optik if you
+        # want the whole machinery to work
+        if 'default' in opt_dict:
+            if OPTPARSE_FORMAT_DEFAULT and 'help' in opt_dict:
+                opt_dict['help'] += ' [current: %default]'
+            del opt_dict['default']
         args = ['--' + opt_name]
-        if opt_dict.has_key('short'):
+        if 'short' in opt_dict:
             self._short_options[opt_dict['short']] = opt_name
             args.append('-' + opt_dict['short'])
             del opt_dict['short']
         available_keys = set(self._optik_parser.option_class.ATTRS)
+        # cleanup option definition dict before giving it to optik
         for key in opt_dict.keys():
             if not key in available_keys:
                 opt_dict.pop(key)
@@ -442,8 +466,12 @@ class OptionsManagerMixIn(object):
         """write a man page for the current configuration into the given
         stream or stdout
         """
-        generate_manpage(self._optik_parser, pkginfo,
-                         section, stream=stream or sys.stdout)
+        self._monkeypatch_expand_default()
+        try:
+            generate_manpage(self._optik_parser, pkginfo,
+                             section, stream=stream or sys.stdout)
+        finally:
+            self._unmonkeypatch_expand_default()
         
     # initialization methods ##################################################
 
@@ -463,6 +491,8 @@ class OptionsManagerMixIn(object):
         """
         if config_file is None:
             config_file = self.config_file
+        if config_file is not None:
+            config_file = expanduser(config_file)
         if config_file and exists(config_file):
             self._config_parser.read([config_file])
         elif not self.quiet:
@@ -511,13 +541,7 @@ class OptionsManagerMixIn(object):
 
         return additional arguments
         """
-        # monkey patch optparse to deal with our default values
-        try:
-            expand_default_backup = HelpFormatter.expand_default
-            HelpFormatter.expand_default = expand_default
-        except AttributeError:
-            # python < 2.4: nothing to be done
-            pass
+        self._monkeypatch_expand_default()
         try:
             if args is None:
                 args = sys.argv[1:]
@@ -533,9 +557,7 @@ class OptionsManagerMixIn(object):
                     setattr(config, attr, value)
             return args
         finally:
-            if hasattr(HelpFormatter, 'expand_default'):
-                # unpatch optparse to avoid side effects
-                HelpFormatter.expand_default = expand_default_backup
+            self._unmonkeypatch_expand_default()
 
 
     # help methods ############################################################
@@ -546,13 +568,31 @@ class OptionsManagerMixIn(object):
                             title=title.capitalize(),
                             description=description)
         self._optik_parser.add_option_group(group)
+
+    def _monkeypatch_expand_default(self):
+        # monkey patch optparse to deal with our default values
+        try:
+            self.__expand_default_backup = HelpFormatter.expand_default
+            HelpFormatter.expand_default = expand_default
+        except AttributeError:
+            # python < 2.4: nothing to be done
+            pass
+    def _unmonkeypatch_expand_default(self):
+        # remove monkey patch
+        if hasattr(HelpFormatter, 'expand_default'):
+            # unpatch optparse to avoid side effects
+            HelpFormatter.expand_default = self.__expand_default_backup
         
     def help(self):
-        """return the usage string for available options """
-        return self._optik_parser.format_help()
+        """return the usage string for available options """ 
+        self._monkeypatch_expand_default()
+        try:
+            return self._optik_parser.format_help()
+        finally:
+            self._unmonkeypatch_expand_default()
     
 
-class Method:
+class Method(object):
     """used to ease late binding of default method (so you can define options
     on the class using default methods on the configuration instance)
     """
@@ -569,10 +609,9 @@ class Method:
         assert self._inst, 'unbound method'
         return getattr(self._inst, self.method)()
 
-        
-class OptionsProviderMixIn:
-    """Mixin to provide options to an OptionsManager
-    """
+
+class OptionsProviderMixIn(object):
+    """Mixin to provide options to an OptionsManager"""
     
     # those attributes should be overridden
     priority = -1
@@ -784,7 +823,7 @@ class Configuration(ConfigurationMixIn):
         super(Configuration, self).__init__(config_file=config_file, usage=usage, version=version)
 
 
-class OptionsManager2ConfigurationAdapter:
+class OptionsManager2ConfigurationAdapter(object):
     """Adapt an option manager to behave like a
     `logilab.common.configuration.Configuration` instance
     """
